//  Beispiel 16.2: MQTT-Allnet Daten vom Server holen und anzeigen
//
//  Unter Verwendung der IoT32 Brick Library

#include <all_IoT32.h>
#include <all_MQTT.h>


int counter = 0;


void setup()
{
  IoT_Init(true);
  IoT_TerminalWaitInit(true);
  IoT_WLANautoConnect(true);                                        // Verbindung, ggf. gespeicherte, über WLAN herstellen
  Serial.println("Looking for NTP-Server...");
  IoT_NTPinit();                                                    // NTP-Timeserver initialisieren
  while (not(IoT_NTPvalid()))
  {
    delay(100);
  }
  if (IoT_NTPeventAvail)                                        // Zeitevent triggern
  {
    IoT_NTPprintEvent(IoT_NTPcurrentEvent);
    IoT_NTPeventAvail = false;
  }
  IoT_DisplayClear(24);                                             // OLED Display löschen (24 Punkt Schrift)
  IoT_DisplayAlignText(TEXT_ALIGN_CENTER);                          // Zentrierte Darstellung des Textes
  IoT_DisplayDrawText(64, 5, "Suche mqtt");
  IoT_DisplayDrawText(64, 34, "Server...");
  IoT_DisplayUpdate();                                              // OLED-Anzeige aktualisieren
  Serial.println("");
  String DeviceID = "Brick-" + hexlong(rnd(2147483647));
  Serial.println("Device-ID = " + DeviceID);
  MQTT_Init("iot.allnet.de", 1883, DeviceID, "open", "Allnet"); // iot.allnet.de = IPval("212.18.29.166") ("open",  "Allnet")
  MQTT_Connect("esp/#");

  Serial.print(t_TerminalClearScreen());
  MQTT_ConnectTrace = false;
}

void loop()
{
  IoT_Idle();

  if (IoT_Keypress())                                               // Wenn Taster gedrückt wird, Konfiguration zeigen
  {
    IoT_DisplayWLANstatus();
    IoT_DisplayUpdate();                                            // OLED-Anzeige aktualisieren
    Serial.println(IoT_NTPdatetime() + " (" + IoT_NTPdaylightSavingTime(true) + ")");
    Serial.print("WiFi is ");
    Serial.print(IoT_WLANconnected() ? "connected" : "not connected");
    Serial.println(". Uptime: " + IoT_WLANuptime(true) + " seit " + IoT_WLANstartDatetime());
    IoT_WaitNoKeypress();
  }
  else
  {
    IoT_Idle();
    if (counter % 25 == 0) //Infos seriell nicht zu oft ausgeben (ca. alle 2 Sekunden)
    {
      if (IoT_NTPeventAvail)                                        // Zeitevent triggern
      {
        IoT_NTPprintEvent(IoT_NTPcurrentEvent);
        IoT_NTPeventAvail = false;
      }
    }
    
    IoT_DisplayClear(16);                                           // OLED Display löschen (16 Punkt Schrift)
    IoT_DisplayAlignText(TEXT_ALIGN_CENTER);                        // Zentrierte Darstellung des Textes
    if (IoT_WLANinited)
    {
      IoT_DisplayClear(24);                                         // OLED Display löschen (24 Punkt Schrift)
      if (MQTT_Client.connected())
      {
        IoT_DisplayAlignText(TEXT_ALIGN_CENTER);                    // Zentrierte Darstellung des Textes
        IoT_DisplayDrawText(64, 5, "Verbunden");
        IoT_DisplayDrawText(64, 34, "mit mqtt");
      }
      else
      {
        IoT_DisplayAlignText(TEXT_ALIGN_CENTER);                    // Zentrierte Darstellung des Textes
        IoT_DisplayDrawText(64, 5, "Suche mqtt");
        IoT_DisplayDrawText(64, 34, "Server...");
      }
      IoT_DisplayUpdate();                                          // OLED-Anzeige aktualisieren
    }
    else
    {
      IoT_DisplayClear(24);                                         // OLED Display löschen (24 Punkt Schrift)
      IoT_DisplayAlignText(TEXT_ALIGN_CENTER);                      // Zentrierte Darstellung des Textes
      IoT_DisplayDrawText(64, 5, "Kein");
      IoT_DisplayDrawText(64, 34, "WLAN");
    }
  }
  IoT_DisplayUpdate();                                              // OLED-Anzeige aktualisieren

  bool success = MQTT_HandleClient();                               // Verbindung aufrecht erhalten, ggf. neu aufbauen

  if (MQTT_EventAvail)
  {
    MQTT_GetNextEvent();
    if (not(MQTT_AllnetParseEventCurrent()))
    {
      Serial.print("Time: "      + cleanasc(String(MQTT_EventCurrentTime)));
      Serial.println(", Topic: " + cleanasc(String(MQTT_EventCurrentTopic)));
      Serial.println("Message: " + String(MQTT_EventCurrentMessage));
    }
  }
  else
  {
    String eol = t_TerminalClearEndOfLine();
    t_TerminalRead();                                                     // Reset-Anforderung prüfen
    Serial.print(t_TerminalCursorHome());
    Serial.print("Verbindungsstatus mit Server '" + MQTT_ServerURL + "': ");
    IoT_DisplayClear(24);                                                 // OLED Display löschen (24 Punkt Schrift)
    if (MQTT_Client.connected())
    {
      IoT_DisplayAlignText(TEXT_ALIGN_CENTER);                            // Zentrierte Darstellung des Textes
      IoT_DisplayDrawText(64, 5, "Verbunden");
      IoT_DisplayDrawText(64, 34, "mit mqtt");
      Serial.println("Verbindung hergestellt" + eol);
    }
    else
    {
      IoT_DisplayAlignText(TEXT_ALIGN_CENTER);                            // Zentrierte Darstellung des Textes
      IoT_DisplayDrawText(64, 5, "Suche mqtt");
      IoT_DisplayDrawText(64, 34, "Server...");
      Serial.println("Server wird gesucht" + eol);
    }
    IoT_DisplayUpdate();                                                  // OLED-Anzeige aktualisieren
    String gc = chr(176) + "C";
    Serial.println(eol);
    Serial.print  (fillcenter("- Temperaturen ", "-", 80));
    Serial.println(eol);
    Serial.print  ("Hamburg     = " + strrealform(MQTT_AllnetTemperaturHamburg / 100.0,      32, 2, 1, false, false) + gc + spc(19));
    Serial.println("Berlin      = " + strrealform(MQTT_AllnetTemperaturBerlin / 100.0,       32, 2, 1, false, false) + gc + eol);
    Serial.print  ("Frankfurt   = " + strrealform(MQTT_AllnetTemperaturFrankfurt / 100.0,    32, 2, 1, false, false) + gc + spc(19));
    Serial.println("Stuttgart   = " + strrealform(MQTT_AllnetTemperaturStuttgart / 100.0,    32, 2, 1, false, false) + gc + eol);
    Serial.print  ("Hannover    = " + strrealform(MQTT_AllnetTemperaturHannover / 100.0,     32, 2, 1, false, false) + gc + spc(19));
    Serial.println("Muenchen    = " + strrealform(MQTT_AllnetTemperaturMuenchen / 100.0,     32, 2, 1, false, false) + gc + eol);
    Serial.print  ("Koeln       = " + strrealform(MQTT_AllnetTemperaturKoeln / 100.0,        32, 2, 1, false, false) + gc + spc(19));
    Serial.println("Duesseldorf = " + strrealform(MQTT_AllnetTemperaturDuesseldorf / 100.0,  32, 2, 1, false, false) + gc + eol);
    Serial.print  ("Nuernberg   = " + strrealform(MQTT_AllnetTemperaturNuernberg / 100.0,    32, 2, 1, false, false) + gc + spc(19));
    Serial.println("Dresden     = " + strrealform(MQTT_AllnetTemperaturDresden / 100.0,      32, 2, 1, false, false) + gc + eol);
    Serial.println(eol);
    Serial.print  (fillcenter(" Aktien- und Waehrungskurse ", "-", 80));
    Serial.println(eol);
    Serial.print  ("DAX         = " + strrealform(MQTT_AllnetFinanceDaxVal,    32, 6, 2, true, false) + spc(16));
    Serial.println("EUR         = " + strrealform(MQTT_AllnetFinanceUSDVal,    32, 6, 2, true, false) + " USD" + eol);
    Serial.print  ("MDAX        = " + strrealform(MQTT_AllnetFinanceMDaxVal,   32, 6, 2, true, false) + spc(16));
    Serial.println("EUR         = " + strrealform(MQTT_AllnetFinanceCHFVal,    32, 6, 2, true, false) + " CHF" + eol);
    Serial.print  ("ESTX50      = " + strrealform(MQTT_AllnetFinanceEStx50Val, 32, 6, 2, true, false) + spc(16));
    Serial.println("EUR         = " + strrealform(MQTT_AllnetFinanceGBPVal,    32, 6, 2, true, false) + " GBP" + eol);
    Serial.print  ("Gold        = " + strrealform(MQTT_AllnetFinanceGoldVal,   32, 6, 2, true, false) + " EUR" + spc(12));
    Serial.println("EUR         = " + strrealform(MQTT_AllnetFinanceJPYVal,    32, 6, 2, true, false) + " JPY" + eol);
    Serial.print  ("Oel         = " + strrealform(MQTT_AllnetFinanceOelVal,    32, 6, 2, true, false) + " EUR" + spc(12));
    Serial.println("EUR         = " + strrealform(MQTT_AllnetFinanceCNYVal,    32, 6, 2, true, false) + " CNY" + eol);
    Serial.print  ("Bitcoin     = " + strrealform(MQTT_AllnetFinanceBTCVal,    32, 6, 2, true, false) + " EUR" + spc(12));
    Serial.println("Ethereum    = " + strrealform(MQTT_AllnetFinanceETHVal,    32, 6, 2, true, false) + " EUR" + eol);
    Serial.println(eol);
    Serial.print  (fillcenter(" Sonstige ", "-", 80));
    Serial.println(eol);
    delay(1000);                                                          // 1 Sekunde warten
  }

  counter++;
}

